\version "2.16.2"

\pointAndClickOff

#(define delete-beams-for-certain-durations
;; Deletes @code{'BeamEvent} from @code{'articulations} if the duration of
;; the @code{'NoteEvent} is greater than or equal to a quarter.
;; LIMITATION:
;; The @code{Beam} is deleted only if the first and the last beamed notes are
;; greater than or equal to a quarter.
  (lambda (music)
    (if (music-is-of-type? music 'note-event)
        (let* ((dur (ly:duration-log (ly:music-property music 'duration))))
           (if (<= dur 2)
             (set! (ly:music-property music 'articulations)
                (remove
                  (lambda (a) (music-is-of-type? a 'beam-event))
                  (ly:music-property music 'articulations))))
           music) 
        music)))
      
noBeamForLongerNotes =
#(define-music-function (parser location m)(ly:music?)
" Maps @code{delete-beams-for-certain-durations} over @var{m} "
(music-map delete-beams-for-certain-durations m))

%% c/p from lily-library.scm
%% Why not public?
#(define (list-minus a b)
  "Return list of elements in A that are not in B."
  (lset-difference eq? a b))
  
#(define modify-beaming
  (lambda (grob)
    (let* ((all-stems (ly:grob-array->list (ly:grob-object grob 'stems)))
           (visible-stems (ly:grob-array->list (ly:grob-object grob 'normal-stems)))
           ;; not visible stems
           (stx (list-minus all-stems visible-stems)))

   %% Affect beaming for mixed notes and rests
   %% For debugging, uncomment modified 'thickness and 'color
     (map
      (lambda (x y z)
        (let* ((beaming-x (ly:grob-property x 'beaming))
               (beaming-y (ly:grob-property y 'beaming))
               (all-stems-length (length all-stems))
               )
        (cond 
          ;;RED
              ((and (member x visible-stems)
                    (member y stx)
                    (or (member z visible-stems) (member z stx))
                    (not (equal? x (first all-stems))))
                 ;(ly:grob-set-property! x 'thickness 10)
                 ;(ly:grob-set-property! x 'color red)
                 (ly:grob-set-property! x 'beaming (cons (car beaming-x) (list 0))))
          ;;CYAN 
              ((and (member x visible-stems)
                    (member y visible-stems)
                    (member z stx)
                    (equal? x (first all-stems)))
                 ;(ly:grob-set-property! y 'thickness 10)
                 ;(ly:grob-set-property! y 'color cyan)
                 (ly:grob-set-property! y 'beaming (cons (car beaming-y) (list 0))))
          ;;BLUE
              ((and (member x stx)
                    (member y visible-stems)
                    (member z visible-stems))
                 ;(ly:grob-set-property! y 'thickness 10)
                 ;(ly:grob-set-property! y 'color blue)
                 (ly:grob-set-property! y 'beaming (cons (list 0) (cdr beaming-y))))
              (else #f))))
       all-stems
       (cdr all-stems)
       (cddr all-stems))

  ;; print only one beam over rests
  (map
    (lambda (x)
      (ly:grob-set-property! x 'beaming (cons (list 0) (list 0))))
    stx))))
    
modifyBeaming = \override Beam #'after-line-breaking = #modify-beaming

% include paper layout
% uncomment the version we want to use
\include "building-blocks-a4.ily"
%\include "building-blocks-beamer.ily"

% include global layout definitions
\include "layout.ily"

%{ function to 
   - print the exercise number
   - print the virtual time signature
   - insert a line break
%}
newExercise = 
#(define-music-function (parser location numerator) (number?)
    #{
      \break
      \mark \default
      \clef percussion
      % denominator is set within \patterns as a function of the current layer
      \set Staff.timeSignatureFraction = #(cons numerator denominator) 
    #})


% include the actual (generated) musical content
\include "building-blocks-patterns.ily"

% not implemented yet
\include "building-blocks-numbers.ily"


% Prepare the five versions of the pattern
% No need to insert \modifyBeaming into the first three ones
I =  {
  \noBeamForLongerNotes
  % Use the \patterns function and pass the denominator as argument
  \patterns 2
}

II =  {
  \noBeamForLongerNotes
  % Change the half notes to crotchets
  \shiftDurations #1 #0
  % make them use double space
  \scaleDurations 2/1
  % use the modified pattern with "4" as denominator
  \patterns 4
}

III =  {
  \noBeamForLongerNotes
  \shiftDurations #2 #0
  \scaleDurations 4/1
  \patterns 8
}

IV =  {
  \modifyBeaming
  \noBeamForLongerNotes
  \shiftDurations #3 #0
  \scaleDurations 8/1
  \patterns 16
}

V =  {
  \modifyBeaming
  \noBeamForLongerNotes
  \scaleDurations 16/1
  \shiftDurations #4 #0
  \patterns 32
}


% Define our score structure
\score {
  % Several layers in parallel
  <<
    % Five DrumStaff instances with their corresponding patterns
    \new RhythmicStaff  { \I }
    \new RhythmicStaff  { \II }
    \new RhythmicStaff  { \III }
    \new RhythmicStaff  { \IV }
    \new RhythmicStaff  { \V }
    % One  additional context without staff for the numbers
    %\new Dynamics \numbers
  >>
  % This actually triggers creating a print layout
  \layout { }
}
